'..::INI-Modul::..
'------------------------
' Autor: Nikolas Gbel
' Date: 05.07.09
'
' Email: comnik@gmx.net
'------------------------

 SuperStrict 
 
 Rem
  bbdoc:   INI Modul.
  about:   Modul to handle with *.ini files.
  License: Permission is hereby granted, free of charge, to any person obtaining a copy of
            this software And associated documentation files (the "Software"), To deal in"
             the Software without restriction, including without limitation the rights To
             use, copy, modify, merge, publish, distribute, sublicense, And/Or sell copies of
             the Software, And To permit persons To whom the Software is furnished To do so,
             subject To the following conditions:
           
             The author Nikolas Gbel has To be by name mentioned."
 End Rem 
 Module comnik.inimod

 ModuleInfo "Description: With this INI Module, you'll be able to parse and write .ini Files in a very Easy Way!"
 ModuleInfo "Version: 1.0"
 ModuleInfo "Author: Nikolas Gbel"
 ModuleInfo "License: Freeware"
 ModuleInfo "Copyright: Nikolas Gbel"

 Import BRL.FileSystem
 Import BRL.Retro

 
 Global ini_list:TList = CreateList()

'-------------------------

 Rem
  bbdoc:   INI Class.
  about:   Contains the data array for *.ini files.
 End Rem 
 Type TIni
  Field filename:String
  Field data:String[]
 End Type 

'--------------------------

  Rem 
   bbdoc:   Opens a .ini File.
   returns: A TIni Object Handle.
   about:   A .ini File is opend And it's value is stored in the 'data' array. 
  End Rem 
  Function OpenIni:TIni(filename:String)
    'Create New Ini Object
    Local ini:TIni = New TIni
    ini.filename = filename
    ini.data = New String[0]
    ListAddLast(ini_list,ini)
  
    'Read the Ini File into the array
    Local _INI:TStream 
    Local index:Int = 0
    _INI = OpenFile(ini.filename) 
   
    While Not Eof(_INI)
     ini.data = ini.data[..Len(ini.data)+1]
     ini.data[index] = ReadLine(_INI)
     index:+ 1
    Wend 
    CloseFile(_INI)  
  
    Return ini
  End Function 
 
  Rem 
   bbdoc:   Reads a specified value from the .ini file.
   returns: A 'String' variable.
   about:   The 'data' array is searched for the given value.
  End Rem
  Function GetValue:String(ini:TIni,_field:String,_value:String)
    Local i:Int
     'Search the 'data' Array for the given field
     For i=0 To Len(ini.data)
      ini.data[i] = Replace(ini.data[i]," ","")
      If Left(ini.data[i],1) = "["
       If Mid(ini.data[i],2,Len(_field)) = _field
        'Search the Field for the given value
        Local j:Int  
        For j=i+1 To Len(ini.data)
         ini.data[j] = Replace(ini.data[j]," ","")
         If Left(ini.data[j],1) = "[" Then Return 0;
         If Left(ini.data[j],Instr(ini.data[j],"=")-1) = _value 
          Local operator:Int = Instr(ini.data[j],"=")
          Local searched_value:String = Mid(ini.data[j],operator+1,Len(ini.data[j])-operator)
          If Instr(searched_value,"'") Then searched_value = Replace(searched_value,"'","")
          Return searched_value
         End If  
        Next 
       End If
      End If 
     Next 
  End Function  

  Rem 
   bbdoc:   Reads a specified value from the .ini file and wraps it into a integer variable.
   returns: A 'Int' variable.
   about:   The 'data' array is searched for the given value.
  End Rem 
  Function GetInt:Int(ini:TIni,_field:String,_value:String)
    Return Int(GetValue(ini,_field,_value))
  End Function 

  Rem 
   bbdoc:   Reads a specified value from the .ini file and wraps it into a float variable.
   returns: A 'Float' variable.
   about:   The 'data' array is searched for the given value.
  End Rem 
  Function GetFloat:Float(ini:TIni,_field:String,_value:String)
    Return Float(GetValue(ini,_field,_value))
  End Function 
 
  Rem
   bbdoc:   Sets the Value of a speciefied key in the .ini File.
   returns: True, if succesfull.
   about:   Searchs the .ini Fiel for the given key. - And sets it's Value.
             - If the given key DOESE'NT exists, it'll be created.
             - If the key-value already EXISTS, the value 'll be overwritten.
             - If no key is given, a new key with value = 0 'll be created
  End Rem
  Function SetKeyValue:Byte(ini:TIni,_field:String,_key:String,_value:String="") 
   Local i:Int
     'Search the 'data' Array for the given field
     For i=0 To Len(ini.data)
      ini.data[i] = Replace(ini.data[i]," ","")
      If Left(ini.data[i],1) = "["
       If Mid(ini.data[i],2,Len(_field)) = _field
        'Search the Field for the given value
        Local j:Int  
        For j=i+1 To Len(ini.data)
         ini.data[j] = Replace(ini.data[j]," ","")
         'If the ky doesent exists: Create it!
         If Left(ini.data[j],1) = "[" Then Return AddKey(ini,_field,_key,_value);
         'Else
         Local operator:Int = Instr(ini.data[j],"=")
         If Left(ini.data[j],operator-1) = _key 
         Local tmp:String = Left(ini.data[j],operator-1)

            ini.data[j] = tmp
            ini.data[j]:+ "="+_value 

          Return True 
         End If  
        Next 
       End If
      End If 
     Next 
   Return False 
  End Function 

  Rem 
   bbdoc:    Adds a new field to the .ini File.
   returns:  True, if succesfull.
  End Rem 
  Function AddField:Byte(ini:TIni,name:String) 
    Local last_element:Int = Len(ini.data)
    ini.data = ini.data[..last_element+1]
    ini.data[last_element] = "["+name+"]"       
  End Function 

  Rem
   bbdoc:   Adds a new key + it's value to the .ini File.
   returns: True, if succesfull.
  End Rem
  Function AddKey:Byte(ini:TIni,_field:String,name:String,_value:String="")
    Local i:Int
     'Search the 'data' Array for the given field
    Local elem:String 
     For elem = EachIn ini.data
      elem = Replace(elem," ","")
      If Left(elem,1) = "["
       If Mid(elem,2,Len(_field)) = _field
       
         Local last:String[] = ini.data[i+1..]
         Local first:String[] = ini.data[..i+1]
         Local new_elem:Int = Len(first)
         first = first[..new_elem+1]
         first[new_elem] = name+"="+_value
         ini.data = first[..]+last[..]         
       
       End If
      End If
      i:+ 1
     Next
  End Function 

 Rem 
   bbdoc:   Saves the .ini File.
   returns: True, if succesfull.
   about:   Use this Function after all changes to the .ini File are done.
 End Rem 
 Function SaveIni:Byte(ini:TIni)
   Local out:TStream = OpenStream(ini.filename)
    If Not out Return False
    Local elem:String 
     For elem = EachIn ini.data
      WriteLine(out,elem)
     Next  
    CloseStream(out)
   Return True
 End Function 






